
	list p=16f877

	include "p16f877.inc"	;standard microchip header
	include "kenmac.inc"	;useful macros

        __CONFIG   _CP_ALL&_DEBUG_OFF&_WRT_ENABLE_OFF&_CPD_OFF&_LVP_OFF&_BODEN_OFF&_PWRTE_ON&_WDT_OFF&_XT_OSC

TOC0		EQU	0X20	;num of tracks
TOC1		EQU	0X21	;trk1 start minutes
TOC2		EQU	0X22	;trk1 start seconds
TOC3		EQU	0X23	;trk1 start frames
TOC4		EQU	0X24	;etc
;..................
;..................
TOC75		EQU	0X6B


CD_STATUS	EQU	0X6C	;to store cd rom status
TEMPC		EQU	0X6D	;temp storage
TEMPD		EQU	0X6E	;temp storage
TEMPE		EQU	0X6F	;temp storage


;0X70 FOR DEBUGGER USE

BCOUNT_LOW	EQU	0X71	;to store bcount low reg
FSR_STORE	EQU	0X72	;temp store for fsr
TRACK		EQU	0X73	;store track number (single trigger mode)
SPIN_UP_COUNT	EQU	0X74	;counter for spin up option
AD_TRACK	EQU	0X75	;store track number (multi trigger mode)
PARAMETER1	EQU	0X76	;parameter in command packet
PARAMETER2	EQU	0X77	; ditto
PARAMETER3	EQU	0X78	; ditto
PARAMETER4	EQU	0X79	; ditto
PARAMETER5	EQU	0X7A	; ditto
PARAMETER6	EQU	0X7B	; ditto
DATA_LOW	EQU	0X7C	; lsbyte to load data lines
DATA_HIGH	EQU	0X7D	; msbyte to load data lines
COUNTER		EQU	0X7E	; used in delay routines and general temp storage
REPEAT		EQU	0X7F	; used in delay routines


MS0		EQU	0XA0
;.............
;.............
MS31		EQU	0XBF

;======== define input/ output pins etc..


#define		DD0		PORTC,1		; .16 bidirectional data lines
#define		DD1		PORTC,3
#define		DD2		PORTD,1
#define		DD3		PORTD,3
#define		DD4		PORTC,5
#define		DD5		PORTC,7
#define		DD6		PORTD,5
#define		DD7		PORTD,7

#define		DD8		PORTD,6
#define		DD9		PORTD,4
#define		DD10	PORTC,6
#define		DD11	PORTC,4
#define		DD12	PORTD,2
#define		DD13	PORTD,0
#define		DD14	PORTC,2
#define		DD15	PORTC,0


#define		DD0_TEMP	TEMPC,1		; for temp storage
#define		DD1_TEMP	TEMPC,3
#define		DD2_TEMP	TEMPD,1
#define		DD3_TEMP	TEMPD,3
#define		DD4_TEMP	TEMPC,5
#define		DD5_TEMP	TEMPC,7
#define		DD6_TEMP	TEMPD,5
#define		DD7_TEMP	TEMPD,7

#define		DD8_TEMP	TEMPD,6
#define		DD9_TEMP	TEMPD,4
#define		DD10_TEMP	TEMPC,6
#define		DD11_TEMP	TEMPC,4
#define		DD12_TEMP	TEMPD,2
#define		DD13_TEMP	TEMPD,0
#define		DD14_TEMP	TEMPC,2
#define		DD15_TEMP	TEMPC,0

#define		DIOW_	PORTE,2		; write strobe
#define		DIOR_	PORTE,1		; read strobe

#define		DA0		PORTA,3		;cd rom register address
#define		DA1		PORTA,5		;cd rom register address
#define		DA2		PORTA,2		;cd rom register address

#define		CS0_	PORTA,1		;cd rom register address
#define		CS1_	PORTB,5		;cd rom register address

;input
#define		INTRQ	PORTE,0		;interrupt line


;some other useful outputs

#define		LED		PORTA,4		;led (A4 sink only so will be active low)
#define		OUT_PLAY	PORTB,7		;high when playing back(after spin up delay)
#define		OUT_IMMED	PORTB,6		;high immediately after trigger until end of play

;these inputs to portb use internal pull up

#define		TRIG		PORTB,0		;trigger input for single trigger mode
#define		TRIG_TYPE	PORTB,1		;option : 	1 = no, 0 = nc
#define		TRIG_REPEAT	PORTB,2		;option : 	1 = ignore repeated triggers till end of trk,
									;	  		0 = accept repeated trigs
#define		RANDOM		PORTB,3		;option : 	1 = random trk, 0 = sequential
#define		SPINUP		PORTB,4		;option : 	1 = cd drive will power down after inactivity timeout,
									;	  		0 = keep disc spinning constantly




	constant  max_trks = .24
	
	
;the following define range of ad conversion results for multi trigger mode

	constant  adtrk1 = .10
	constant  adtrk2 = .40
	constant  adtrk3 = .80
	constant  adtrk4 = .116
	constant  adtrk5 = .150
	constant  adtrk6 = .185
	constant  adtrk7 = .210
	constant  adtrk8 = .240


	org	0x00
	nop				;for debugger
	goto	start


	org	0x04		; should never get here!
	goto	start

	
msec_delay		;gives approximate 1ms delay at 4Mhz

	movlw	.249
	movwf	COUNTER
msec_loop
	nop
	decfsz	COUNTER,F
	goto	msec_loop
	return

;=========================================================================================
	
xmsec_delay		;multiples of 1ms delay approx
	movwf	REPEAT
xmsec_loop
	call	msec_delay
	decfsz	REPEAT,F
	goto	xmsec_loop
	return

;===================================================================================



store_lsb	
	
	clrf	INDF
	btfsc	DD0_TEMP
	bsf	INDF,0
	btfsc	DD1_TEMP
	bsf	INDF,1
	btfsc	DD2_TEMP
	bsf	INDF,2
	btfsc	DD3_TEMP
	bsf	INDF,3
	btfsc	DD4_TEMP
	bsf	INDF,4
	btfsc	DD5_TEMP
	bsf	INDF,5
	btfsc	DD6_TEMP
	bsf	INDF,6
	btfsc	DD7_TEMP
	bsf	INDF,7

	return

;==============================================================================

store_msb	

	clrf	INDF
	btfsc	DD8_TEMP
	bsf	INDF,0
	btfsc	DD9_TEMP
	bsf	INDF,1
	btfsc	DD10_TEMP
	bsf	INDF,2
	btfsc	DD11_TEMP
	bsf	INDF,3
	btfsc	DD12_TEMP
	bsf	INDF,4
	btfsc	DD13_TEMP
	bsf	INDF,5
	btfsc	DD14_TEMP
	bsf	INDF,6
	btfsc	DD15_TEMP
	bsf	INDF,7

	return

;==============================================================================

write_data	

	clrf	PORTC
	clrf	PORTD
	

	btfsc	DATA_LOW,0
	bsf	DD0
	btfsc	DATA_LOW,1
	bsf	DD1
	btfsc	DATA_LOW,2
	bsf	DD2
	btfsc	DATA_LOW,3
	bsf	DD3
	btfsc	DATA_LOW,4
	bsf	DD4
	btfsc	DATA_LOW,5
	bsf	DD5
	btfsc	DATA_LOW,6
	bsf	DD6
	btfsc	DATA_LOW,7
	bsf	DD7

	btfsc	DATA_HIGH,0
	bsf	DD8
	btfsc	DATA_HIGH,1
	bsf	DD9
	btfsc	DATA_HIGH,2
	bsf	DD10
	btfsc	DATA_HIGH,3
	bsf	DD11
	btfsc	DATA_HIGH,4
	bsf	DD12
	btfsc	DATA_HIGH,5
	bsf	DD13
	btfsc	DATA_HIGH,6
	bsf	DD14
	btfsc	DATA_HIGH,7
	bsf	DD15

	nop
	bcf	DIOW_
	nop
	bsf	DIOW_


	return


;===============================================================================

read_data		

	bcf	DIOR_
	nop
	movfw	PORTC
	movwf	TEMPC
	movfw	PORTD
	movwf	TEMPD
	bsf	DIOR_		

	return

;=================================================================================
	

read_status		;read cd rom status reg

	bcf	CS0_
	bsf	CS1_
	bsf	DA0
	bsf	DA1
	bsf	DA2		;set up address

	call	data_input
	call	read_data

	movlw	CD_STATUS
	movwf	FSR
	call	store_lsb


	return



;============================================================================



read_bcount_low			;read cd rom byte count low register


	bcf	CS0_
	bsf	CS1_
	bcf	DA0
	bcf	DA1
	bsf	DA2		;set up address

	call	data_input

	call	read_data

	movlw	BCOUNT_LOW
	movwf	FSR
	call	store_lsb


	return




;===========================================================================


write_bcount_low		;write to cd rom byte count low register

	bcf	CS0_
	bsf	CS1_
	bcf	DA0
	bcf	DA1
	bsf	DA2		;set up address

	call	data_output

	movfw	BCOUNT_LOW
	movwf	DATA_LOW
	clrf	DATA_HIGH	;only need lsb
	call	write_data


	return



;===========================================================================


cd_busy			;only returns when drq=0 and bsy=0



	call	read_status

	btfsc	CD_STATUS,3		;DRQ=0?
	goto	cd_busy
	btfsc	CD_STATUS,7		;BSY=0?
	goto	cd_busy

	return

;===========================================================================


cd_ready_for_packet	;only returns when drq = 1 and bsy = 0	



	call	read_status

	btfss	CD_STATUS,3		;DRQ=1?
	goto	cd_ready_for_packet
	btfsc	CD_STATUS,7		;BSY=0?
	goto	cd_ready_for_packet

	return




;============================================================================





select_device0			;use cd rom dev/head register to select device0 (master)	 

	bcf	CS0_
	bsf	CS1_
	bcf	DA0
	bsf	DA1
	bsf	DA2		;set address for dev/head reg

	
	call	data_output

; DEV BIT IS DD4 ( 0 for master, would be 1 for slave)

	clrf	DATA_LOW

	clrf	DATA_HIGH
	call	write_data


	return

;===========================================================================


features_reg		;use cd rom features reg to select no dma and no ovl

	bcf	CS0_
	bsf	CS1_
	bsf	DA0
	bcf	DA1
	bcf	DA2		;set address for features reg

	call	data_output

;clear DMA(DD0) and OVL(DD1) bits

	clrf	DATA_LOW
	clrf	DATA_HIGH
	call	write_data

	return

;===========================================================================


send_ata_cmd		;sends ata cmd 0xa0 - atapi packet cmd
	
	bcf	CS0_
	bsf	CS1_
	bsf	DA0
	bsf	DA1
	bsf	DA2		;set address for command reg

	call	data_output

	movlw	0xa0
	movwf	DATA_LOW
	clrf	DATA_HIGH
	call	write_data


	return

;==============================================================================

data_input		;set tris portc/d for data input


	bank1
	movlw	0xff
	movwf	TRISC
	movwf	TRISD		
	bank0
	
	return



;============================================================================

data_output		;set tris portc/d for data output


	bank1
	clrf	TRISC
	clrf	TRISD		
	bank0

	return

;===========================================================================

datareg_out		;set address lines for data reg and tris = out

	bcf	CS0_
	bsf	CS1_
	bcf	DA0
	bcf	DA1
	bcf	DA2		;set address for data reg

	call	data_output

	return

;===========================================================================

datareg_in		;set address lines for data reg and tris = input


	bcf	CS0_
	bsf	CS1_
	bcf	DA0
	bcf	DA1
	bcf	DA2		;set address for data reg

	call	data_input

	return

;============================================================================


start

	clrf	PORTA
	clrf	PORTB
	clrf	PORTC
	clrf	PORTD
	clrf	PORTE

	
	clrf	TRACK
	clrf	AD_TRACK

	bank1

	movlw	b'00001110'
	movwf	ADCON1			;a0 ANALOGUE, REST digital
	movlw	b'00000001'
	movwf	TRISA			;a0 in, rest out
	movlw	b'00011111'
	movwf	TRISB			; b7,b6,b5 out, rest in
	movlw	b'00000001'
	movwf	TRISE			;e0 in

	movlw	b'00000000'
	movwf	OPTION_REG		;enable pull-ups

	
	bank0	

	movlw	b'01000001'
	movwf	ADCON0

	bsf	DIOR_
	bsf	DIOW_

	clrf	T2CON		;1:1 pre/post

	movlw	.250
	call	xmsec_delay
	movlw	.250
	call	xmsec_delay
	bsf	LED		;led off( gives initial 1/2 sec flash)


	movlw	.250
	call	xmsec_delay
	movlw	.250
	call	xmsec_delay
	bcf	LED
	movlw	.250
	call	xmsec_delay
	movlw	.250
	call	xmsec_delay
	bsf	LED
	movlw	.250
	call	xmsec_delay
	movlw	.250
	call	xmsec_delay
	bcf	LED
	movlw	.250
	call	xmsec_delay
	movlw	.250
	call	xmsec_delay
	bsf	LED
	movlw	.250
	call	xmsec_delay
	movlw	.250
	call	xmsec_delay
	bcf	LED
	movlw	.250
	call	xmsec_delay
	movlw	.250
	call	xmsec_delay
	bsf	LED

		
	call	select_device0	
	call	cd_busy
	call	features_reg	
get_num_trks

	call	exe_read_num_trks
	xorlw	0x00
	btfsc	STATUS,Z
	goto	get_num_trks_ok

get_num_trks_error
	
	bcf	OUT_IMMED
	bcf	OUT_PLAY
	bcf	LED	
	movlw	.250
	call	xmsec_delay
	bsf	LED			;short flash
	movlw	.250
	call	xmsec_delay
	goto	get_num_trks

get_num_trks_ok			;disc now loaded and toc read ok


	
	decf	TOC0,W
	bank1
	movwf	PR2			;tmr2 used for trk selection if random play
	bank0
	bsf	T2CON,TMR2ON

	bcf	LED
	movlw	.250
	call	xmsec_delay
	movlw	.250
	call	xmsec_delay		;long flash
	bsf	LED

outer_loop

	btfss	TRIG_TYPE
	goto	nc_outer_loop

no_outer_loop		;wait till high

	btfss	OUT_PLAY
	goto	start_playback
	btfss	TRIG
	goto	outer_spinup_check
		
	movlw	.25		;debounce
	call	xmsec_delay
	goto	inner_loop
	
nc_outer_loop		;wait till low

	btfsc	TRIG
	goto	outer_spinup_check

	movlw	.25		;debounce
	call	xmsec_delay

inner_loop

	btfss	OUT_PLAY
	goto	start_playback
	btfss	TRIG_TYPE
	goto	nc_inner_loop

no_inner_loop


	bsf		ADCON0,GO
	nop
	btfsc	ADCON0,GO
	goto	$-1

	sfltl	ADRESH, adtrk8		;macros
	goto	ad_none_select
	sfltl	ADRESH, adtrk7
	goto	ad_trk8_select
	sfltl	ADRESH, adtrk6
	goto	ad_trk7_select
	sfltl	ADRESH, adtrk5
	goto	ad_trk6_select
	sfltl	ADRESH, adtrk4
	goto	ad_trk5_select
	sfltl	ADRESH, adtrk3
	goto	ad_trk4_select
	sfltl	ADRESH, adtrk2
	goto	ad_trk3_select
	sfltl	ADRESH, adtrk1
	goto	ad_trk2_select
ad_trk1_select

	sfnel	AD_TRACK, .1
	goto	ad_same_track		;stops repeated triggering
	movlw	.1
	movwf	AD_TRACK
	goto	ad_is_trk_valid
	
ad_trk2_select
	sfnel	AD_TRACK, .2
	goto	ad_same_track
	movlw	.2
	movwf	AD_TRACK
	goto	ad_is_trk_valid
ad_trk3_select
	sfnel	AD_TRACK, .3
	goto	ad_same_track
	movlw	.3
	movwf	AD_TRACK
	goto	ad_is_trk_valid	
ad_trk4_select
	sfnel	AD_TRACK, .4
	goto	ad_same_track
	movlw	.4
	movwf	AD_TRACK
	goto	ad_is_trk_valid
ad_trk5_select
	sfnel	AD_TRACK, .5
	goto	ad_same_track
	movlw	.5
	movwf	AD_TRACK
	goto	ad_is_trk_valid
ad_trk6_select
	sfnel	AD_TRACK, .6
	goto	ad_same_track
	movlw	.6
	movwf	AD_TRACK
	goto	ad_is_trk_valid
ad_trk7_select
	sfnel	AD_TRACK, .7
	goto	ad_same_track
	movlw	.7
	movwf	AD_TRACK
	goto	ad_is_trk_valid
ad_trk8_select
	sfnel	AD_TRACK, .8
	goto	ad_same_track
	movlw	.8
	movwf	AD_TRACK
ad_is_trk_valid
	sfltef	AD_TRACK, TOC0		
	goto	ad_same_track		
	movfw	AD_TRACK
	movwf	PARAMETER1
	goto	send_playback_command


ad_none_select
	clrf	AD_TRACK

ad_same_track				
	btfsc	TRIG
	goto	inner_spinup_check
	goto	start_playback



nc_inner_loop

	btfss	TRIG			
	goto	inner_spinup_check

start_playback

	btfss	RANDOM
	goto	sequential_play
random_play
	movfw	TMR2		
	addlw	0x01
	movwf	PARAMETER1	
	goto	send_playback_command
sequential_play
	incf	TRACK,F
	sfgtf	TRACK,TOC0		
	goto	track_in_range
	movlw	0x01			
	movwf	TRACK

track_in_range
	movfw	TRACK
	movwf	PARAMETER1

send_playback_command

	bcf	LED		
	bsf	OUT_IMMED		
	call	exe_play_trk	
	xorlw	0x00
	btfss	STATUS,Z
	goto	get_num_trks_error


playback_ok

	movlw	.25
	call	xmsec_delay		

	call	exe_read_audio_status	;return val in w
	movwf	TEMPC
	sfltl	TEMPC, .128		;error if > 127
	goto	get_num_trks_error
	bsf		OUT_PLAY		;now switch on the delayed output

playback_wait
	btfss	TRIG_REPEAT		;wait till end of trk before next trigger?
	goto	outer_loop		;no

;;;yes
	call	exe_read_audio_status	;return val in w
	movwf	TEMPC
	sfltl	TEMPC, .128		;error if > .127
	goto	get_num_trks_error
	sfel	TEMPC, .17		;returns .17 if playing
	goto	outer_loop		;play finished
	goto	playback_wait		;still playing


inner_spinup_check

	movlw	.20
	call	xmsec_delay

	call	exe_read_audio_status	;return val in w
	movwf	TEMPC
	sfltl	TEMPC, .128		;error if > .127
	goto	get_num_trks_error
	sfel	TEMPC, .17			;playing?
	goto	inner_spinup_not_playing	;no
	clrf	SPIN_UP_COUNT			;yes
	goto	inner_loop

inner_spinup_not_playing

	bsf		LED			;led off	
	bcf		OUT_IMMED		;outputs off
	bcf		OUT_PLAY

	incfsz	SPIN_UP_COUNT,F		;spinup executed once every (256 * 20ms)
	goto	inner_loop
	btfsc	SPINUP			;ignore if option not selected
	goto	inner_loop
	call	exe_spinup		;returns 0x00 in w if ok
	xorlw	0x00
	btfss	STATUS,Z
	goto	get_num_trks_error
	goto	inner_loop


outer_spinup_check

	movlw	.20
	call	xmsec_delay

	call	exe_read_audio_status	;return val in w
	movwf	TEMPC
	sfltl	TEMPC, .128		;error if > 127
	goto	get_num_trks_error
	sfel	TEMPC, .17		;playing?
	goto	outer_spinup_not_playing
	clrf	SPIN_UP_COUNT
	goto	outer_loop

outer_spinup_not_playing

	bsf	LED			;led off	
	bcf	OUT_IMMED		;outputs off
	bcf	OUT_PLAY

	incfsz	SPIN_UP_COUNT,F
	goto	outer_loop
	btfsc	SPINUP
	goto	outer_loop		;ignore if option not selected
	call	exe_spinup		;returns 0x00 in w if ok
	xorlw	0x00
	btfss	STATUS,Z
	goto	get_num_trks_error
	goto	outer_loop


exe_read_num_trks	

	call	toc_cmd

	btfss	CD_STATUS,0	;chk bit(error if 1)
	goto	ernt_ok
	call	get_error
	return
ernt_ok
	retlw	0x00

;========================================================================




exe_play_trk		

	movfw	PARAMETER1
	addwf	PARAMETER1,F
	addwf	PARAMETER1,F
	decf	PARAMETER1,F
	decf	PARAMETER1,F

	movfw	PARAMETER1
	addlw	TOC0
	movwf	FSR		;beginning address of data for required track

	movfw	INDF
	movwf	PARAMETER1	;now holds track start mm
	incf	FSR,F
	movfw	INDF
	movwf	PARAMETER2	;track start ss
	incf	FSR,F
	movfw	INDF
	movwf	PARAMETER3	;track start ff
	incf	FSR,F
	movfw	INDF
	movwf	PARAMETER4	;track end mm (next track start mm)
	incf	FSR,F
	movfw	INDF
	movwf	PARAMETER5	;track end ss (next track start ss)
	incf	FSR,F
	movfw	INDF
	movwf	PARAMETER6	;track end ff (next track start ff)


	call	play_cmd

	movlw	0x00
	btfsc	CD_STATUS,0	;chk bit(error if 1)
	call	get_error
	return




;========================================================================


exe_read_audio_status	

	call	subchannel_cmd

	btfss	CD_STATUS,0	;chk bit(error if 1)
	goto	eras_ok
	call	get_error
	return
eras_ok


	movlw	(MS0 + 1)	;this is where audio status is stored
	movwf	FSR
	movfw	INDF

	return


;========================================================================




exe_spinup	

	call	seek_cmd

	movlw	0x00
	btfsc	CD_STATUS,0	;chk bit(error if 1)
	call	get_error

	return



toc_cmd


	
	call	cd_busy
	movlw	0xff
	movwf	BCOUNT_LOW		
	call	write_bcount_low

	call	cd_busy
	call	send_ata_cmd

	call	cd_ready_for_packet
	
	call	datareg_out

;bytes 1 and 0 - 0x02, 0x43

	movlw	0x02
	movwf	DATA_HIGH
	movlw	0x43
	movwf	DATA_LOW
	call	write_data

;bytes 3 and 2 - 0x00, 0x00

	movlw	0x00
	movwf	DATA_HIGH
	movlw	0x00
	movwf	DATA_LOW
	call	write_data

;bytes 5 and 4 - 0x00, 0x00

	movlw	0x00
	movwf	DATA_HIGH
	movlw	0x00
	movwf	DATA_LOW
	call	write_data

;bytes 7 and 6 - 0x00, 0x00

	movlw	0x00
	movwf	DATA_HIGH
	movlw	0x00
	movwf	DATA_LOW
	call	write_data

;bytes 9 and 8 - 0x00, (max_trks x 8) + 4

	movlw	0x00
	movwf	DATA_HIGH
	movlw	((max_trks + 1) * 8) + 4	;this is where the TOTAL number of bytes is limited
	movwf	DATA_LOW
	call	write_data

;bytes 11 and 10 - 0x00, 0x00

	movlw	0x00
	movwf	DATA_HIGH
	movlw	0x00
	movwf	DATA_LOW
	call	write_data

	clrf	COUNTER
	movlw	TOC0			;start address for toc data store	
	movwf	FSR_STORE
rtoc_receive
	btfss	INTRQ
	goto	$-1
	call	read_status		;this changes fsr hence need for fsr_store

	btfss	CD_STATUS,3		;DRQ=1?
	goto	rtoc_end		;not 1 so no more data

	call	read_bcount_low		;this changes fsr hence need for fsr_store
	call	datareg_in

	bcf	STATUS,C
	rrf	BCOUNT_LOW,F		;divide by 2 , returns 2 bytes at a time
	movfw	FSR_STORE
	movwf	FSR
rtoc_loop
	
	call	read_data

	incf	COUNTER,F

	sfel	COUNTER,0X02
	goto	rtoc1
	call	store_msb		;num of tracks
	incf	FSR,F
	goto	rtoc_cont
rtoc1
	sfel	COUNTER,0X05
	goto	rtoc2
	call	store_msb		;MM
	incf	FSR,F
	goto	rtoc_cont
rtoc2
	sfel	COUNTER,0X06
	goto	rtoc_cont
	call	store_lsb		;SS
	incf	FSR,F
	call	store_msb		;FF
	incf	FSR,F
	movlw	0x02
	movwf	COUNTER		;reset to 2
	

rtoc_cont
	decfsz	BCOUNT_LOW,F
	goto	rtoc_loop

	movfw	FSR
	movwf	FSR_STORE
	goto	rtoc_receive

rtoc_end


	sfgtl	TOC0, max_trks
	goto 	dec_lead_out_frames
	movlw	max_trks
	movwf	TOC0
	return

dec_lead_out_frames

	movfw	TOC0		;num of tracks(n)

	movwf	COUNTER		;temp storage
	addwf	COUNTER,F
	addwf	COUNTER,F
	incf	COUNTER,F


	movfw	COUNTER
	addlw	TOC0
	movwf	FSR
	movfw	INDF		;lead out mm
	movwf	TEMPC

	incf	FSR,F
	movfw	INDF		;lead out ss
	movwf	TEMPD

	incf	FSR,F
	movfw	INDF		;lead out ff
	movwf	TEMPE

;decrement ff then update values in toc data

	sfz	TEMPE
	goto	endff_notzero
	sfz	TEMPD
	goto	endss_notzero

	decf	TEMPC,F		;ff and ss zero		
	movlw	.59
	movwf	TEMPD
	movlw	.74
	movwf	TEMPE		
	goto	update_msf
endss_notzero
	decf	TEMPD,F		;ff zero
	movlw	.74
	movwf	TEMPE
	goto	update_msf
endff_notzero
	decf	TEMPE,F

;fsr pointing to ff

update_msf
	movfw	TEMPE
	movwf	INDF
	decf	FSR,F
	movfw	TEMPD
	movwf	INDF
	decf	FSR,F
	movfw	TEMPC
	movwf	INDF

	return


;===========================================================================

play_cmd

	call	cd_busy

	call	send_ata_cmd

	call	cd_ready_for_packet
	
	call	datareg_out

;bytes 1 and 0 - 0x00, 0x47

	clrf	DATA_HIGH
	movlw	0x47
	movwf	DATA_LOW
	call	write_data

;bytes 3 and 2 - PARAMETER1, 0x00  

	movfw	PARAMETER1
	movwf	DATA_HIGH
	clrf	DATA_LOW
	call	write_data

;bytes 5 and 4 - PARAMETER3, PARAMETER2

	movfw	PARAMETER3
	movwf	DATA_HIGH
	movfw	PARAMETER2
	movwf	DATA_LOW
	call	write_data
	
;bytes 7 and 6 - PARAMETER5,PARAMETER4
	movfw	PARAMETER5
	movwf	DATA_HIGH
	movfw	PARAMETER4
	movwf	DATA_LOW
	call	write_data

;bytes 9 and 8 - 0x00,PARAMETER6

	clrf	DATA_HIGH
	movfw	PARAMETER6
	movwf	DATA_LOW
	call	write_data

;bytes 11 and 10 - 0x00, 0x00

	clrf	DATA_HIGH
	clrf	DATA_LOW
	call	write_data

	btfss	INTRQ
	goto	$-1
	call	read_status

	return



;============================================================================


subchannel_cmd		

	call	cd_busy
	movlw	0xff
	movwf	BCOUNT_LOW	
	call	write_bcount_low


	call	cd_busy
	call	send_ata_cmd

	call	cd_ready_for_packet
	
	call	datareg_out

;bytes 1 and 0 - 0x02, 0x42

	movlw	0x02
	movwf	DATA_HIGH
	movlw	0x42
	movwf	DATA_LOW
	call	write_data

;bytes 3 and 2 - 0x01, 0x40

	movlw	0x01
	movwf	DATA_HIGH
	movlw	0x40
	movwf	DATA_LOW
	call	write_data

;bytes 5 and 4 - 0x00, 0x00

	clrf	DATA_HIGH
	clrf	DATA_LOW
	call	write_data

;bytes 7 and 6 - 0x00, 0x00

	clrf	DATA_HIGH
	clrf	DATA_LOW
	call	write_data

;bytes 9 and 8 - 0x00, 0x14

	clrf	DATA_HIGH
	movlw	0x14
	movwf	DATA_LOW
	call	write_data

;bytes 11 and 10 - 0x00, 0x00

	clrf	DATA_HIGH
	clrf	DATA_LOW
	call	write_data

	movlw	MS0			;start address of where data is to be stored	
	movwf	FSR_STORE
subchannel_receive
	btfss	INTRQ
	goto	$-1
	call	read_status

	btfss	CD_STATUS,3		;DRQ=1?
	goto	subchannel_end		;not 1 so no more data

	call	read_bcount_low
	call	datareg_in

	bcf	STATUS,C
	rrf	BCOUNT_LOW,F		;/2
	movfw	FSR_STORE
	movwf	FSR
subchannel_loop
	
	call	read_data
	
	call	store_lsb
	incf	FSR,F
	call	store_msb
	incf	FSR,F
	decfsz	BCOUNT_LOW,F
	goto	subchannel_loop

	movfw	FSR
	movwf	FSR_STORE
	goto	subchannel_receive

subchannel_end
	

	return
	

get_error	;uses request sense command to get error codes


	call	cd_busy
	movlw	0xff
	movwf	BCOUNT_LOW	
	call	write_bcount_low

	call	cd_busy
	call	send_ata_cmd

	call	cd_ready_for_packet
	
	call	datareg_out

;bytes 1 and 0 - 0x00, 0x03

	movlw	0x00
	movwf	DATA_HIGH
	movlw	0x03
	movwf	DATA_LOW
	call	write_data

;bytes 3 and 2 - 0x00, 0x00

	clrf	DATA_HIGH
	clrf	DATA_LOW
	call	write_data

;bytes 5 and 4 - 0x00, 0x12

	movlw	0x00
	movwf	DATA_HIGH
	movlw	0x12
	movwf	DATA_LOW
	call	write_data

;bytes 7 and 6 - 0x00, 0x00

	clrf	DATA_HIGH
	clrf	DATA_LOW
	call	write_data

;bytes 9 and 8 - 0x00, 0x00

	clrf	DATA_HIGH
	clrf	DATA_LOW
	call	write_data

;bytes 11 and 10 - 0x00, 0x00

	clrf	DATA_HIGH
	clrf	DATA_LOW
	call	write_data

	movlw	MS0		
	movwf	FSR_STORE
request_sense_receive
	btfss	INTRQ
	goto	$-1
	call	read_status

	btfss	CD_STATUS,3		;DRQ=1?
	goto	request_sense_end		;not 1 so no more data

	call	read_bcount_low
	call	datareg_in

	bcf	STATUS,C
	rrf	BCOUNT_LOW,F		;/2
	movfw	FSR_STORE
	movwf	FSR
request_sense_loop
	
	call	read_data
	
	call	store_lsb
	incf	FSR,F
	call	store_msb
	incf	FSR,F
	decfsz	BCOUNT_LOW,F
	goto	request_sense_loop

	movfw	FSR
	movwf	FSR_STORE
	goto	request_sense_receive

request_sense_end
	

	bank1
	movfw	(MS0 + .2)	;this is where error code is stored
	bank0
	andlw	0x0f	
	iorlw	0x80

	return
	



;============================================================================

seek_cmd		

	call	cd_busy	
	call	send_ata_cmd

	call	cd_ready_for_packet
	
	call	datareg_out

;bytes 1 and 0 - 0x00, 0x2b

	movlw	0x00
	movwf	DATA_HIGH
	movlw	0x2b
	movwf	DATA_LOW
	call	write_data

;bytes 3 and 2 - 0x00, 0x00  

	clrf	DATA_HIGH
	clrf	DATA_LOW
	call	write_data

;bytes 5 and 4 - .150, 0x00

	movlw	.150	;start of trk1 (in logical block address)
	movwf	DATA_HIGH
	movlw	0x00
	movwf	DATA_LOW
	call	write_data
	
;bytes 7 and 6 - 0x00, 0x00	

	clrf	DATA_HIGH
	clrf	DATA_LOW
	call	write_data

;bytes 9 and 8 - 0x00,0x00

	movlw	0x00
	movwf	DATA_HIGH
	movlw	0x00
	movwf	DATA_LOW
	call	write_data

;bytes 11 and 10 - 0x00, 0x00

	clrf	DATA_HIGH
	clrf	DATA_LOW
	call	write_data
;packet sent


	btfss	INTRQ
	goto	$-1
	call	read_status


	return


	end


